function getPrototypeOmnitrixItem(colorScore, phase, stage) {
    if (phase === 1 && stage === 2) {
        let omnitrix10KMap = {
            0: 'alienevo:prototype_omnitrix_10k',
            1: 'alienevo:prototype_omnitrix_10k_white',
            2: 'alienevo:prototype_omnitrix_10k_light_gray',
            3: 'alienevo:prototype_omnitrix_10k_gray',
            4: 'alienevo:prototype_omnitrix_10k_black',
            5: 'alienevo:prototype_omnitrix_10k_brown',
            6: 'alienevo:prototype_omnitrix_10k_red',
            7: 'alienevo:prototype_omnitrix_10k_orange',
            8: 'alienevo:prototype_omnitrix_10k_yellow',
            9: 'alienevo:prototype_omnitrix_10k_green',
            10: 'alienevo:prototype_omnitrix_10k_cyan',
            11: 'alienevo:prototype_omnitrix_10k_light_blue',
            12: 'alienevo:prototype_omnitrix_10k_blue',
            13: 'alienevo:prototype_omnitrix_10k_purple',
            14: 'alienevo:prototype_omnitrix_10k_magenta',
            15: 'alienevo:prototype_omnitrix_10k_pink'
        };
        return omnitrix10KMap[colorScore] || 'alienevo:prototype_omnitrix_10k';
    }
    
    const colorSuffixes = [
        '', '_white', '_light_gray', '_gray', '_black', '_brown',
        '_red', '_orange', '_yellow', '_green', '_cyan', '_light_blue',
        '_blue', '_purple', '_magenta', '_pink'
    ];
    
    let baseItemId = 'alienevo:prototype_omnitrix';
    
    if (phase === 1) {
        if (stage === 0) {
            baseItemId += '_phase1';
        } else if (stage === 1) {
            baseItemId += '_phase2';
        }
    }
    
    if (colorScore >= 0 && colorScore < colorSuffixes.length) {
        baseItemId += colorSuffixes[colorScore];
    }
    
    return baseItemId;
}

function createAndGivePrototypeOmnitrix(entity) {
    let MAX_PLAYLISTS = 10;
    let SLOTS_PER_PLAYLIST = 10;
    let OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';
    
    let allDNA = {};
    let storedAlienCount = 0;

    let storedItemName = entity.persistentData.getString('storedItemName') || '';

    let alienNamespaceData = {};

    for (let playlist = 1; playlist <= MAX_PLAYLISTS; playlist++) {
        allDNA[playlist] = {};
        for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
            let alienKey = `alienevo.alien_${playlist}_${slot}`;
            let alienId = entity.persistentData.getInt(alienKey);

            if (alienId && alienId !== 0) {
                let alienInfo = global[`alienevo_alien_${alienId}`];
                let alienName = "Unknown";
                let alienNamespace = "alienevo_aliens";
                
                if (alienInfo) {
                    let alienFullName = alienInfo[0];
                    
                    if (alienFullName.includes(':')) {
                        let parts = alienFullName.split(':');
                        alienNamespace = parts[0];
                        alienName = parts[1];
                    } else {
                        alienName = alienFullName;
                    }
                    
                    alienNamespaceData[alienId] = {
                        namespace: alienNamespace,
                        name: alienName
                    };
                } else {
                    alienName = getAlienName(alienId);
                }
                
                allDNA[playlist][slot] = {
                    alienId: alienId,
                    alienName: alienName
                };
                storedAlienCount++;
                entity.persistentData.putInt(alienKey, 0);
            }
        }
    }

    let username = entity.getGameProfile().getName();

    let colorProp = palladium.getProperty(entity, 'AlienEvo.Color') || 'default';
    let primaryColorProp = palladium.getProperty(entity, 'AlienEvo.PrimaryColor') || 'default';
    let secondaryColorProp = palladium.getProperty(entity, 'AlienEvo.SecondaryColor') || 'default';
    
    let omnitrixCycleValue = 1;
    let omnitrixCycleProp = palladium.getProperty(entity, OMNITRIX_PROPERTY_KEY);
    if (omnitrixCycleProp) {
        try {
            omnitrixCycleValue = parseInt(omnitrixCycleProp);
        } catch (e) {
            omnitrixCycleValue = 1;
        }
    }

    function getScore(scoreName) {
        let scoreResult = entity.server.runCommandSilent(`scoreboard players get ${username} ${scoreName}`);
        return isNaN(scoreResult) ? 0 : parseInt(scoreResult);
    }

    let basePrimaryScore = getScore('AlienEvo.BasePrimary');
    let baseSecondaryScore = getScore('AlienEvo.BaseSecondary');
    let baseTertiaryScore = getScore('AlienEvo.BaseTertiary');
    let buttonsScore = getScore('AlienEvo.Buttons');
    let coreSideScore = getScore('AlienEvo.CoreSide');
    let coreTopScore = getScore('AlienEvo.CoreTop');
    let dialInnerScore = getScore('AlienEvo.DialInner');
    let dialOuterScore = getScore('AlienEvo.DialOuter');
    let tubesScore = getScore('AlienEvo.Tubes');
    let phase10kScore = getScore('AlienEvo.10kPhase');
    let stage10kScore = getScore('AlienEvo.10kStage');

    let currentCode = entity.persistentData.getString('code') || '000000';
    let currentPosition = entity.persistentData.getString('position') || '0';
    let masterTargetCode = entity.persistentData.getString('masterTargetCode') || '';
    let selfDestructTargetCode = entity.persistentData.getString('selfDestructTargetCode') || '';

    let currentPlaylist = entity.persistentData.getInt('current_playlist') || 1;
    let currentAlienSlot = entity.persistentData.getInt('current_alien_slot') || 1;

    let colorScore = 0;
    if (colorProp && colorProp !== 'default') {
        let colorMap = {
            'white': 1,
            'lightgray': 2,
            'gray': 3,
            'black': 4,
            'brown': 5,
            'red': 6,
            'orange': 7,
            'yellow': 8,
            'green': 9,
            'cyan': 10,
            'lightblue': 11,
            'blue': 12,
            'purple': 13,
            'magenta': 14,
            'pink': 15
        };
        colorScore = colorMap[colorProp] || 0;
    }

    let hasMastercontrol = entity.tags.contains('AlienEvo.MasterControl');
    let hasBaseForm = entity.tags.contains('AlienEvo.BaseForm');

    let prototypeOmnitrixItem = getPrototypeOmnitrixItem(colorScore, phase10kScore, stage10kScore);
    let prototypeOmnitrix;

    let displayNbt = {};
    if (storedItemName && storedItemName !== '') {
        displayNbt = { Name: storedItemName };
        entity.persistentData.remove('storedItemName');
    }
    
    let nbtData = {
        storedAllDNA: JSON.stringify(allDNA),
        storedAlienCount: storedAlienCount,
        storedAlienNamespaces: JSON.stringify(alienNamespaceData),
        storedCurrentPlaylist: currentPlaylist,
        storedCurrentAlienSlot: currentAlienSlot,
        storedOmnitrixCycle: omnitrixCycleValue,
        storedColor: colorProp,
        storedPrimaryColor: primaryColorProp,
        storedSecondaryColor: secondaryColorProp,
        storedBasePrimaryScore: basePrimaryScore,
        storedBaseSecondaryScore: baseSecondaryScore,
        storedBaseTertiaryScore: baseTertiaryScore,
        storedButtonsScore: buttonsScore,
        storedCoreSideScore: coreSideScore,
        storedCoreTopScore: coreTopScore,
        storedDialInnerScore: dialInnerScore,
        storedDialOuterScore: dialOuterScore,
        storedTubesScore: tubesScore,
        stored10kPhaseScore: phase10kScore,
        stored10kStageScore: stage10kScore,
        storedCode: currentCode,
        storedPosition: currentPosition,
        storedMasterTargetCode: masterTargetCode,
        storedSelfDestructTargetCode: selfDestructTargetCode,
        storedMastercontrol: hasMastercontrol,
        storedBaseForm: hasBaseForm,
        display: displayNbt
    };

    prototypeOmnitrix = Item.of(prototypeOmnitrixItem);
    prototypeOmnitrix.setNbt(nbtData);

    entity.give(prototypeOmnitrix);

    palladium.setProperty(entity, 'AlienEvo.Color', 'default');
    palladium.setProperty(entity, 'AlienEvo.PrimaryColor', 'default');
    palladium.setProperty(entity, 'AlienEvo.SecondaryColor', 'default');
    palladium.setProperty(entity, OMNITRIX_PROPERTY_KEY, 1);

    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kPhase 0`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kStage 0`);

    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.BasePrimary -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.BaseSecondary -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.BaseTertiary -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.Buttons -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.CoreSide -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.CoreTop -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.DialInner -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.DialOuter -1`);
    entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.Tubes -1`);

    entity.persistentData.remove('code');
    entity.persistentData.remove('position');
    entity.persistentData.remove('masterTargetCode');
    entity.persistentData.remove('selfDestructTargetCode');
    entity.server.runCommandSilent(`scoreboard players set ${username} code_position 0`);
    palladium.setProperty(entity, 'code', "000000");

    entity.persistentData.remove('current_playlist');
    entity.persistentData.remove('current_alien_slot');

    entity.tags.remove('AlienEvo.MasterControl');
    entity.tags.remove('AlienEvo.BaseForm');

    superpowerUtil.removeSuperpower(entity, new ResourceLocation(`alienevo:prototype_omnitrix`));
}

function applyAllDNAFromPrototypeOmnitrix(entity) {
    let OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';
    
    let mainHandItem = entity.getMainHandItem();
    if (!mainHandItem.id.startsWith('alienevo:prototype_omnitrix')) {
        return;
    }

    let itemColor = 'default';
    let itemId = mainHandItem.id;
    
    if (itemId !== 'alienevo:prototype_omnitrix') {
        if (itemId.includes('_10k')) {
            let colorMap = {
                'alienevo:prototype_omnitrix_10k': 'default',
                'alienevo:prototype_omnitrix_10k_white': 'white',
                'alienevo:prototype_omnitrix_10k_light_gray': 'lightgray',
                'alienevo:prototype_omnitrix_10k_gray': 'gray',
                'alienevo:prototype_omnitrix_10k_black': 'black',
                'alienevo:prototype_omnitrix_10k_brown': 'brown',
                'alienevo:prototype_omnitrix_10k_red': 'red',
                'alienevo:prototype_omnitrix_10k_orange': 'orange',
                'alienevo:prototype_omnitrix_10k_yellow': 'yellow',
                'alienevo:prototype_omnitrix_10k_green': 'green',
                'alienevo:prototype_omnitrix_10k_cyan': 'cyan',
                'alienevo:prototype_omnitrix_10k_light_blue': 'lightblue',
                'alienevo:prototype_omnitrix_10k_blue': 'blue',
                'alienevo:prototype_omnitrix_10k_purple': 'purple',
                'alienevo:prototype_omnitrix_10k_magenta': 'magenta',
                'alienevo:prototype_omnitrix_10k_pink': 'pink'
            };
            itemColor = colorMap[itemId] || 'default';
            
            let username = entity.getGameProfile().getName();
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kPhase 1`);
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kStage 2`);
        } else {
            let parts = itemId.split('_');
            if (parts.length > 2) {
                itemColor = parts[parts.length - 1];
                
                if (parts.length > 3 && (parts[parts.length - 2] === "light")) {
                    itemColor = parts[parts.length - 2] + parts[parts.length - 1];
                } else if (itemColor.startsWith("light")) {
                    itemColor = itemColor;
                }
            }
        }
    }

    let nbt = mainHandItem.nbt;
    if (!nbt) {
        if (itemColor !== 'default') {
            palladium.setProperty(entity, 'AlienEvo.Color', itemColor);
        }
        
        if (itemId.includes('_10k')) {
            let username = entity.getGameProfile().getName();
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kPhase 1`);
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kStage 2`);
        }
        
        giveFirstTenAliensRandomOrder(entity);
        entity.setMainHandItem(Item.of('minecraft:air'));
        return;
    }

    let storedAllDNAString = nbt.storedAllDNA || '';
    if (!storedAllDNAString) {
        if (itemColor !== 'default') {
            palladium.setProperty(entity, 'AlienEvo.Color', itemColor);
        }
        
        if (itemId.includes('_10k')) {
            let username = entity.getGameProfile().getName();
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kPhase 1`);
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kStage 2`);
        }
        
        giveFirstTenAliensRandomOrder(entity);
        entity.setMainHandItem(Item.of('minecraft:air'));
        return;
    }

    let displayNbt = nbt.display;
    let itemName = '';
    if (displayNbt && displayNbt.Name) {
        itemName = displayNbt.Name;
        entity.persistentData.putString('storedItemName', itemName);
    }

    let storedAllDNA;
    try {
        storedAllDNA = JSON.parse(storedAllDNAString);
    } catch (e) {
        if (itemColor !== 'default') {
            palladium.setProperty(entity, 'AlienEvo.Color', itemColor);
        }
        
        if (itemId.includes('_10k')) {
            let username = entity.getGameProfile().getName();
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kPhase 1`);
            entity.server.runCommandSilent(`scoreboard players set ${username} AlienEvo.10kStage 2`);
        }
        
        giveFirstTenAliensRandomOrder(entity);
        entity.setMainHandItem(Item.of('minecraft:air'));
        return;
    }

    let alienNamespaceData = {};
    try {
        if (nbt.storedAlienNamespaces) {
            alienNamespaceData = JSON.parse(nbt.storedAlienNamespaces);
        }
    } catch (e) {
        alienNamespaceData = {};
    }

    for (let playlist in storedAllDNA) {
        for (let slot in storedAllDNA[playlist]) {
            let alienKey = `alienevo.alien_${playlist}_${slot}`;
            let alienData = storedAllDNA[playlist][slot];

            if (alienData && alienData.alienId) {
                let alienId = alienData.alienId;
                
                entity.persistentData.putInt(alienKey, alienId);
                
                if (alienData.alienName) {
                    entity.persistentData.putString(`alienevo.alien_name_${alienId}`, alienData.alienName);
                }
                
                if (alienNamespaceData && alienNamespaceData[alienId]) {
                    let namespace = alienNamespaceData[alienId].namespace || "alienevo_aliens";
                    let name = alienNamespaceData[alienId].name || alienData.alienName || `alien_${alienId}`;
                    
                    let alienInfo = [`${namespace}:${name}`];
                    
                    global[`alienevo_alien_${alienId}`] = alienInfo;
                }
            }
        }
    }
    let username = entity.getGameProfile().getName();

    let storedColor = nbt.storedColor || itemColor;
    let storedPrimaryColor = nbt.storedPrimaryColor || 'default';
    let storedSecondaryColor = nbt.storedSecondaryColor || 'default';
    let storedOmnitrixCycle = nbt.storedOmnitrixCycle || 1;
    
    palladium.setProperty(entity, 'AlienEvo.Color', storedColor);
    palladium.setProperty(entity, 'AlienEvo.PrimaryColor', storedPrimaryColor);
    palladium.setProperty(entity, 'AlienEvo.SecondaryColor', storedSecondaryColor);
    palladium.setProperty(entity, OMNITRIX_PROPERTY_KEY, storedOmnitrixCycle);

    let storedCode = nbt.storedCode || '000000';
    let storedPosition = nbt.storedPosition || '0';
    let storedMasterTargetCode = nbt.storedMasterTargetCode || '';
    let storedSelfDestructTargetCode = nbt.storedSelfDestructTargetCode || '';

    entity.persistentData.putString('code', storedCode);
    entity.persistentData.putString('position', storedPosition);
    entity.persistentData.putString('masterTargetCode', storedMasterTargetCode);
    entity.persistentData.putString('selfDestructTargetCode', storedSelfDestructTargetCode);
    
    entity.server.runCommandSilent(`scoreboard players set ${username} code_position ${parseInt(storedPosition)}`);
    palladium.setProperty(entity, 'code', storedCode);

    let storedCurrentPlaylist = nbt.storedCurrentPlaylist || 1;
    let storedCurrentAlienSlot = nbt.storedCurrentAlienSlot || 1;
    entity.persistentData.putInt('current_playlist', storedCurrentPlaylist);
    entity.persistentData.putInt('current_alien_slot', storedCurrentAlienSlot);

    function setScore(scoreName, scoreValue) {
        entity.server.runCommandSilent(`scoreboard players set ${username} ${scoreName} ${scoreValue}`);
    }

    function getStoredScore(nbtTag, defaultValue) {
        return nbt.contains(nbtTag) ? nbt.getInt(nbtTag) : defaultValue;
    }

    let stored10kPhaseScore = getStoredScore('stored10kPhaseScore', 0);
    let stored10kStageScore = getStoredScore('stored10kStageScore', 0);

    let storedBasePrimaryScore = getStoredScore('storedBasePrimaryScore', -1);
    let storedBaseSecondaryScore = getStoredScore('storedBaseSecondaryScore', -1);
    let storedBaseTertiaryScore = getStoredScore('storedBaseTertiaryScore', -1);
    let storedButtonsScore = getStoredScore('storedButtonsScore', -1);
    let storedCoreSideScore = getStoredScore('storedCoreSideScore', -1);
    let storedCoreTopScore = getStoredScore('storedCoreTopScore', -1);
    let storedDialInnerScore = getStoredScore('storedDialInnerScore', -1);
    let storedDialOuterScore = getStoredScore('storedDialOuterScore', -1);
    let storedTubesScore = getStoredScore('storedTubesScore', -1);

    setScore('AlienEvo.10kPhase', stored10kPhaseScore);
    setScore('AlienEvo.10kStage', stored10kStageScore);
    setScore('AlienEvo.BasePrimary', storedBasePrimaryScore);
    setScore('AlienEvo.BaseSecondary', storedBaseSecondaryScore);
    setScore('AlienEvo.BaseTertiary', storedBaseTertiaryScore);
    setScore('AlienEvo.Buttons', storedButtonsScore);
    setScore('AlienEvo.CoreSide', storedCoreSideScore);
    setScore('AlienEvo.CoreTop', storedCoreTopScore);
    setScore('AlienEvo.DialInner', storedDialInnerScore);
    setScore('AlienEvo.DialOuter', storedDialOuterScore);
    setScore('AlienEvo.Tubes', storedTubesScore);

    if (nbt.storedMastercontrol) entity.tags.add('AlienEvo.MasterControl');
    if (nbt.storedBaseForm) entity.tags.add('AlienEvo.BaseForm');

    entity.setMainHandItem(Item.of('minecraft:air'));
}

function giveFirstTenAliensRandomOrder(entity) {
    let MAX_PLAYLISTS = 10;
    let SLOTS_PER_PLAYLIST = 10;

    let firstTenAlienIds = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];

    for (let i = firstTenAlienIds.length - 1; i > 0; i--) {
        let j = Math.floor(Math.random() * (i + 1));
        [firstTenAlienIds[i], firstTenAlienIds[j]] = [firstTenAlienIds[j], firstTenAlienIds[i]];
    }

    let playlist = 1;
    for (let slot = 1; slot <= firstTenAlienIds.length; slot++) {
        let alienKey = `alienevo.alien_${playlist}_${slot}`;
        let alienId = firstTenAlienIds[slot - 1];
        entity.persistentData.putInt(alienKey, alienId);
    }

    for (let slot = firstTenAlienIds.length + 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
        let alienKey = `alienevo.alien_${playlist}_${slot}`;
        entity.persistentData.putInt(alienKey, 0);
    }

    for (let p = 2; p <= MAX_PLAYLISTS; p++) {
        for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
            let alienKey = `alienevo.alien_${p}_${slot}`;
            entity.persistentData.putInt(alienKey, 0);
        }
    }
    
    entity.persistentData.putInt('current_playlist', 1);
    entity.persistentData.putInt('current_alien_slot', 1);
}

function getAlienName(alienId) {
    let alienInfo = global[`alienevo_alien_${alienId}`];
    if (alienInfo) {
        let fullName = alienInfo[0];
        if (fullName.includes(':')) {
            return fullName.split(':')[1];
        }
        return fullName;
    }
    
    let MAX_PLAYLISTS = 10;
    let SLOTS_PER_PLAYLIST = 10;
    
    for (let p = 1; p <= MAX_PLAYLISTS; p++) {
        for (let s = 1; s <= SLOTS_PER_PLAYLIST; s++) {
            let existingData = entity.persistentData.get(`alienevo.alien_name_${p}_${s}`);
            if (existingData && existingData.id === alienId) {
                return existingData.name;
            }
        }
    }

}
StartupEvents.registry('palladium:abilities', (event) => {
    event.create('alienevo:create_all_dna_prototype_omnitrix')
        .icon(palladium.createItemIcon('alienevo:prototype_omnitrix'))
        .addProperty("property", "string", "alienevodnaomnitrixgeneration", "")
        .tick((entity, entry, holder, enabled) => {
            if (enabled) {
                createAndGivePrototypeOmnitrix(entity);
            }
        });

    event.create('alienevo:apply_dna_from_prototype_omnitrix')
        .icon(palladium.createItemIcon('alienevo:prototype_omnitrix'))
        .addProperty("property", "string", "alienevodnaomnitrixapplication", "")
        .tick((entity, entry, holder, enabled) => {
            if (enabled) {
                applyAllDNAFromPrototypeOmnitrix(entity);
            }
        });
});