StartupEvents.registry('palladium:abilities', (event) => {
    const OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle'; 
    const CURRENT_PLAYLIST_KEY = 'current_playlist';
    const CURRENT_ALIEN_SLOT_KEY = 'current_alien_slot';
    const TIMER_OBJECTIVE = 'AlienEvo.Timer';
    
    const TRANSFORM_INCREASE = 500;
    const MAX_TIMER_ALLOWED = 5500;
    
    const DEFAULT_PLAYLIST = 1;
    const DEFAULT_SLOT = 1;
    
    const MAX_PLAYLISTS = 10;
    const MAX_SLOTS = 10;
    
    function getRandomInt(min, max) {
        min = Math.ceil(min);
        max = Math.floor(max);
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }
    
    event.create('alienevo:omnitrix_transform_overheat')
        .icon(palladium.createItemIcon('minecraft:blaze_rod'))
        .addProperty("watch", "string", "prototype", "The name of the Omnitrix and user in the configuration files")
        .addProperty("property", "string", OMNITRIX_PROPERTY_KEY, "Internal property key for the Omnitrix cycle")
        .addProperty("transform_sound", "string", "prototype", "The sound set to use during transformation")
        .addProperty("use_timeout_bubble", "boolean", true, "Whether to show the timeout bubble on detransform")
        
        .firstTick((entity, abilityEntry, abilityHolder, isEnabled) => {
            if (!isEnabled || !entity) return;
            
            const currentTimer = palladium.scoreboard.getScore(entity, TIMER_OBJECTIVE, 0);
            const username = entity.getGameProfile().getName();
            
            const hasMasterControl = entity.tags.contains("AlienEvo.MasterControl");
            
            if (!hasMasterControl && currentTimer >= MAX_TIMER_ALLOWED) {
                entity.tell("§c§lYour Omnitrix is overheated! Wait for it to cool down! (" + 
                    Math.floor((currentTimer - MAX_TIMER_ALLOWED)/20) + " seconds until usable)");
                
                entity.server.runCommandSilent(`execute as ${username} at @s run playsound alienevo:omnitrix_error master @s ~ ~ ~`);
                
                abilityHolder.disable(abilityEntry, true);
                return;
            }
            
            if (!hasMasterControl) {
                palladium.scoreboard.addScore(entity, TIMER_OBJECTIVE, TRANSFORM_INCREASE);
                const newTimer = currentTimer + TRANSFORM_INCREASE;
                
                const remainingBeforeOverheat = MAX_TIMER_ALLOWED - newTimer;
                
                if (remainingBeforeOverheat <= 1000) {
                    entity.tell("§6§lWARNING: Omnitrix heating up! " + 
                        Math.floor(remainingBeforeOverheat/20) + " seconds until overheating!");
                } else {
                    entity.tell("§e§lOmnitrix heat increased. Remaining capacity: " + 
                        Math.floor(remainingBeforeOverheat/20) + " seconds");
                }
            }
            
        })
        
        .lastTick((entity, abilityEntry, abilityHolder, isEnabled) => {
            if (!isEnabled || !entity) return;
            
            const username = entity.getGameProfile().getName();
            
            const hasMasterControl = entity.tags.contains("AlienEvo.MasterControl");
            
            if (hasMasterControl) {
                transformNormal(entity, abilityEntry, username);
            } else {
                transformRandom(entity, abilityEntry, username);
            }
        });
        
    function transformNormal(entity, abilityEntry, username) {
        const watchName = abilityEntry.getPropertyByName('watch');
        const transformSound = abilityEntry.getPropertyByName('transform_sound');
        const useTimeoutBubble = abilityEntry.getPropertyByName('use_timeout_bubble');
        
        const omnitrixPropertyKey = abilityEntry.getPropertyByName('property');
        
        const currentPlaylist = entity.persistentData[CURRENT_PLAYLIST_KEY] || DEFAULT_PLAYLIST;
        const currentAlienSlot = entity.persistentData[CURRENT_ALIEN_SLOT_KEY] || DEFAULT_SLOT;
        
        const alienNumber = entity.persistentData[`alienevo.alien_${currentPlaylist}_${currentAlienSlot}`];
        
        if (alienNumber && alienNumber >= 1) {
            palladium.setProperty(entity, omnitrixPropertyKey, alienNumber);
            
            palladium.setProperty(entity, 'watch', watchName);
            
            palladium.setProperty(entity, 'use_timeout_bubble', useTimeoutBubble);
            
            transformToAlien(entity, alienNumber, username, transformSound);
        }
    }
    
    function transformRandom(entity, abilityEntry, username) {
        const watchName = abilityEntry.getPropertyByName('watch');
        const transformSound = abilityEntry.getPropertyByName('transform_sound');
        const omnitrixPropertyKey = abilityEntry.getPropertyByName('property');
        const useTimeoutBubble = abilityEntry.getPropertyByName('use_timeout_bubble');
        
        const current_playlist = entity.persistentData[CURRENT_PLAYLIST_KEY] || DEFAULT_PLAYLIST;
        const current_slot = entity.persistentData[CURRENT_ALIEN_SLOT_KEY] || DEFAULT_SLOT;
        const selectedAlienNum = entity.persistentData[`alienevo.alien_${current_playlist}_${current_slot}`] || 0;
        
        entity.server.runCommandSilent(`execute as ${username} at @s run playsound alienevo:${transformSound}_transform master @a ~ ~ ~`);
        
        let chance = getRandomInt(1, 100);
        
        if (chance <= 20) {
            let alien_num = 0;
            let attempts = 0;
            const max_attempts = 50;
            
            while (alien_num <= 0 && attempts < max_attempts) {
                let random_playlist = getRandomInt(1, MAX_PLAYLISTS);
                let random_slot = getRandomInt(1, MAX_SLOTS);
                alien_num = entity.persistentData[`alienevo.alien_${random_playlist}_${random_slot}`] || 0;
                
                attempts++;
            }
            
            if (alien_num > 0) {
                palladium.setProperty(entity, omnitrixPropertyKey, alien_num);
                
                palladium.setProperty(entity, 'watch', watchName);
                
                palladium.setProperty(entity, 'use_timeout_bubble', useTimeoutBubble);
                
                transformToAlien(entity, alien_num, username, transformSound);
                
                palladium.setProperty(entity, omnitrixPropertyKey, selectedAlienNum);
            } else {
                if (selectedAlienNum > 0) {
                    palladium.setProperty(entity, omnitrixPropertyKey, selectedAlienNum);
                    
                    palladium.setProperty(entity, 'watch', watchName);
                    
                    palladium.setProperty(entity, 'use_timeout_bubble', useTimeoutBubble);
                    
                    transformToAlien(entity, selectedAlienNum, username, transformSound);
                }
            }
        } else {
            if (selectedAlienNum > 0) {
                palladium.setProperty(entity, omnitrixPropertyKey, selectedAlienNum);
                
                palladium.setProperty(entity, 'watch', watchName);
                
                palladium.setProperty(entity, 'use_timeout_bubble', useTimeoutBubble);
                
                transformToAlien(entity, selectedAlienNum, username, transformSound);
            }
        }
    }
    
    function transformToAlien(entity, alienNumber, username, transformSound) {
        let alienInfo = global[`alienevo_alien_${alienNumber}`];
        if (!alienInfo) {
            return;
        }
        
        let alienFullName = alienInfo[0];
        let alienNamespace = 'alienevo_aliens';
        let alienPath = alienFullName;
        
        if (alienFullName.includes(':')) {
            const parts = alienFullName.split(':');
            alienNamespace = parts[0];
            alienPath = parts[1];
        }
        
        entity.server.runCommandSilent(`execute as ${username} at @s run superpower add ${alienNamespace}:${alienPath}`);
        entity.server.runCommandSilent(`execute as ${username} at @s run superpower replace alienevo:all ${alienNamespace}:${alienPath}`);
        entity.server.runCommandSilent(`execute as ${username} at @s run playsound alienevo:${transformSound}_transform player @s ~ ~ ~ 1 1 1`);
        entity.server.runCommandSilent(`execute as ${username} at @s run superpower add alienevo:transform_bubble`);
    }
});
if (Platform.isClientEnvironment()) {
ClientEvents.init(event => {
    const $KeyMappingRegistry = Java.loadClass("dev.architectury.registry.client.keymappings.KeyMappingRegistry");
    const $KeyMapping = Java.loadClass("net.minecraft.client.KeyMapping");
    const $GLFWKey = Java.loadClass("org.lwjgl.glfw.GLFW");
    for (let i = 1; i <= 10; i++) {
        let keyName = `QUICKCHANGE${i.toString().padStart(2, '0')}`;
        global[keyName] = new $KeyMapping(
            `key.alienevo.key${i}`,
            $GLFWKey.GLFW_KEY_UNKNOWN,
            "key.categories.alienevo"
        );
        
        $KeyMappingRegistry.register(global[keyName]);
    }
})
}