ServerEvents.commandRegistry(event => {
    const { commands: Commands, arguments: Arguments } = event;

    const OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';
    const CURRENT_PLAYLIST_KEY = 'current_playlist';
    const CURRENT_ALIEN_SLOT_KEY = 'current_alien_slot';

    const alienSuggestionProvider = (context, builder) => {
        for (let i = 1; i <= 300; i++) {
            if (global[`alienevo_alien_${i}`]) {
                let alienName = global[`alienevo_alien_${i}`][0];
                builder.suggest(alienName);
            }
        }
        return builder.buildFuture();
    };

    event.register(
        Commands.literal("alienevoplaylist")
        .requires(src => src.hasPermission(2))
        .then(Commands.argument('entity', Arguments.PLAYER.create(event))
            .then(Commands.argument('playlist', Arguments.INTEGER.create(event))
                .then(Commands.argument('slot', Arguments.INTEGER.create(event))
                    .then(Commands.argument('alien', Arguments.RESOURCE_LOCATION.create(event))
                        .suggests(alienSuggestionProvider)
                        .executes(ctx => {
                            let target = Arguments.PLAYER.getResult(ctx, 'entity');
                            let server = ctx.source.getServer();
                            let player = ctx.source.player;
                            const playlist = Arguments.INTEGER.getResult(ctx, "playlist");
                            const slot = Arguments.INTEGER.getResult(ctx, "slot");
                            const alienResource = Arguments.RESOURCE_LOCATION.getResult(ctx, "alien");
                            const alienName = alienResource.toString();

                            const namespace = alienResource.getNamespace();
                            const path = alienResource.getPath();

                            if (playlist < 1 || playlist > 10 || slot < 1 || slot > 10) {
                                ctx.source.sendFailure(Text.of("Invalid input. Playlist and slot must be between 1 and 10."));
                                return 0;
                            }

                            let alienNumber = -1;
                            for (let i = 1; i <= 300; i++) {
                                if (global[`alienevo_alien_${i}`] && 
                                    (global[`alienevo_alien_${i}`][0].toLowerCase() === alienName.toLowerCase() || 
                                     global[`alienevo_alien_${i}`][0].toLowerCase() === path.toLowerCase())) {
                                    alienNumber = i;
                                    break;
                                }
                            }

                            if (alienNumber === -1) {
                                ctx.source.sendFailure(Text.of(`Unknown alien: ${alienName}`));
                                return 0;
                            }

                            let existingPlaylist = null;
                            let existingSlot = null;
                            outerLoop:
                            for (let p = 1; p <= 10; p++) {
                                for (let s = 1; s <= 10; s++) {
                                    let alienInSlot = target.persistentData.getInt(`alienevo.alien_${p}_${s}`);
                                    if (alienInSlot === alienNumber) {
                                        existingPlaylist = p;
                                        existingSlot = s;
                                        break outerLoop;
                                    }
                                }
                            }

                            if (existingSlot !== null) {
                                target.persistentData.putInt(`alienevo.alien_${existingPlaylist}_${existingSlot}`, 0);
                            }

                            target.persistentData.putInt(`alienevo.alien_${playlist}_${slot}`, alienNumber);

                            target.persistentData.putInt(CURRENT_PLAYLIST_KEY, playlist);
                            target.persistentData.putInt(CURRENT_ALIEN_SLOT_KEY, slot);

                            if (target.persistentData.getInt(CURRENT_ALIEN_SLOT_KEY) == slot && 
                                target.persistentData.getInt(CURRENT_PLAYLIST_KEY) == playlist) {
                                palladium.setProperty(target, OMNITRIX_PROPERTY_KEY, alienNumber);
                            }

                            let targetName = target.getGameProfile().getName();
                            let successMessage = `§a§lAlien Set: §r§b${alienName} §7(ID: §f${alienNumber}§7)\n§7Playlist: §f${playlist}§7, Slot: §f${slot}\n§7Player: §f${targetName}`;
                            ctx.source.sendSuccess(Text.of(successMessage), true);

                            return 1;
                        })
                    )
                )
            )
        )
    );
});

ServerEvents.commandRegistry(event => {
    const { commands: Commands, arguments: Arguments } = event;

    const OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';
    const CURRENT_PLAYLIST_KEY = 'current_playlist';
    const CURRENT_ALIEN_SLOT_KEY = 'current_alien_slot';

    event.register(
        Commands.literal("alienevorandomize")
        .requires(src => src.hasPermission(2))
        .then(Commands.argument('entity', Arguments.PLAYER.create(event))
            .then(Commands.argument('playlist', Arguments.INTEGER.create(event))
                .executes(ctx => {
                    let target = Arguments.PLAYER.getResult(ctx, 'entity');
                    const playlist = Arguments.INTEGER.getResult(ctx, "playlist");
                    
                    // Validate input
                    if (playlist < 1 || playlist > 10) {
                        ctx.source.sendFailure(Text.of("Invalid input. Playlist must be between 1 and 10."));
                        return 0;
                    }

                    // Get all available alien IDs from the global variable
                    let availableAliens = [];
                    
                    // Scan through all possible alien IDs including higher, non-sequential ones
                    // Use Object.keys to find all keys that match the pattern
                    let globalKeys = Object.keys(global);
                    for (let key of globalKeys) {
                        if (key.startsWith('alienevo_alien_')) {
                            let idStr = key.replace('alienevo_alien_', '');
                            let id = parseInt(idStr);
                            
                            // Skip ID 0 (alienevo:null)
                            if (id > 0) {
                                availableAliens.push(id);
                            }
                        }
                    }

                    // Check if we have any aliens
                    if (availableAliens.length === 0) {
                        ctx.source.sendFailure(Text.of("No aliens found in database."));
                        return 0;
                    }

                    // Shuffle the array (improved Fisher-Yates shuffle)
                    for (let i = availableAliens.length - 1; i > 0; i--) {
                        // Use crypto random if available for better randomization
                        let j;
                        try {
                            j = Math.floor(Math.random() * (i + 1));
                        } catch(e) {
                            j = Math.floor(Math.random() * (i + 1));
                        }
                        // Swap elements
                        [availableAliens[i], availableAliens[j]] = [availableAliens[j], availableAliens[i]];
                    }
                    
                    // Debug: log the shuffled array to console for troubleshooting
                    console.log("Shuffled aliens for randomization: " + availableAliens.join(", "));

                    // Fill playlist with unique aliens (improved assignment)
                    let uniqueAliensUsed = [];
                    
                    // Clear all slots in the playlist first to avoid any issues
                    for (let slot = 1; slot <= 10; slot++) {
                        target.persistentData.putInt(`alienevo.alien_${playlist}_${slot}`, 0);
                    }
                    
                    // Now assign the shuffled aliens
                    for (let slot = 1; slot <= 10; slot++) {
                        if (availableAliens.length === 0) {
                            break; // Safety check - shouldn't happen
                        }
                        
                        // Take the first alien from our shuffled array
                        let alienNumber = availableAliens[0];
                        
                        // Add alien to the playlist slot
                        target.persistentData.putInt(`alienevo.alien_${playlist}_${slot}`, alienNumber);
                        
                        // Get alien name for logging
                        let alienData = global[`alienevo_alien_${alienNumber}`];
                        let alienName = alienData ? alienData[0] : "Unknown";
                        uniqueAliensUsed.push(`${alienName} (ID: ${alienNumber})`);
                        
                        // Remove this alien from the available list to ensure uniqueness in the playlist
                        availableAliens.shift();
                        
                        // If we've used all aliens, reshuffle what we have
                        if (availableAliens.length === 0 && slot < 10) {
                            // Re-scan global keys to get all aliens again
                            availableAliens = [];
                            for (let key of globalKeys) {
                                if (key.startsWith('alienevo_alien_')) {
                                    let idStr = key.replace('alienevo_alien_', '');
                                    let id = parseInt(idStr);
                                    if (id > 0) {
                                        availableAliens.push(id);
                                    }
                                }
                            }
                            
                            // Shuffle them again
                            for (let i = availableAliens.length - 1; i > 0; i--) {
                                const j = Math.floor(Math.random() * (i + 1));
                                [availableAliens[i], availableAliens[j]] = [availableAliens[j], availableAliens[i]];
                            }
                        }
                    }
                    
                    // Update current playlist to the randomized one
                    target.persistentData.putInt(CURRENT_PLAYLIST_KEY, playlist);
                    target.persistentData.putInt(CURRENT_ALIEN_SLOT_KEY, 1);
                    
                    // Also update current alien if needed
                    let currentAlienNumber = target.persistentData.getInt(`alienevo.alien_${playlist}_1`);
                    if (currentAlienNumber > 0) {
                        palladium.setProperty(target, OMNITRIX_PROPERTY_KEY, currentAlienNumber);
                    }

                    // Format success message
                    let targetName = target.getGameProfile().getName();
                    let successMessage = `§a§lPlaylist Randomized: §r§7Playlist §f${playlist} §7for §f${targetName}`;
                    
                    // Add the aliens to the message
                    if (uniqueAliensUsed.length > 0) {
                        successMessage += `\n§7Aliens in playlist: §f${uniqueAliensUsed.join("§7, §f")}`;
                    }
                    
                    ctx.source.sendSuccess(Text.of(successMessage), true);
                    return 1;
                })
            )
        )
    );
});