BlockEvents.placed('alienevo:omnitrix_workbench', event => {
    let pos = event.block.pos;
    let yaw = event.player.yaw;

    // Adjust yaw to face the same direction as the player
    let adjustedYaw = yaw + 180;
    if (adjustedYaw > 180) {
        adjustedYaw -= 360;
    }

    // Snap the adjusted yaw to the nearest cardinal direction
    if (adjustedYaw >= -135 && adjustedYaw < -45) {
        adjustedYaw = -90; // East
    } else if (adjustedYaw >= -45 && adjustedYaw < 45) {
        adjustedYaw = 0; // South
    } else if (adjustedYaw >= 45 && adjustedYaw < 135) {
        adjustedYaw = 90; // West
    } else {
        adjustedYaw = 180; // North
    }

    // Summon the armor stand with a custom tag
  event.server.runCommandSilent(`summon minecraft:armor_stand ${pos.x + 0} ${pos.y} ${pos.z + 0} {Rotation:[${adjustedYaw}f,0f],Tags:["workbench_stand"],Marker:1b,Invisible:1,NoGravity:1b}`);
});

// Listen for block break to remove the armor stand
BlockEvents.broken('alienevo:omnitrix_workbench', event => {
    let pos = event.block.pos;
    let username = event.player.getGameProfile().getName();
    (`Workbench broken at: ${pos.x}, ${pos.y}, ${pos.z}`);
    
    // Helper function for getting armor stand scores
    function getArmorStandScore(scoreName) {
        let cmd = `scoreboard players get @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] ${scoreName}`;
        let scoreResult = event.server.runCommandSilent(`execute positioned ${pos.x} ${pos.y} ${pos.z} run ${cmd}`);
        return isNaN(scoreResult) ? null : parseInt(scoreResult);
    }

    // Check if armor stand has workbench power
    let hasWorkbenchPower = event.server.runCommandSilent(
        `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} if entity @e[type=minecraft:armor_stand,tag=workbench_stand,sort=nearest,limit=1,distance=..0.8,palladium.power=alienevo:omnitrix_workbench]`
    );

    if (hasWorkbenchPower) {
        ('Found workbench stand with power, starting retrieval process');

        // Get all scores with default values
        let scores = {
            color: getArmorStandScore('AlienEvo.Color') ?? 0,
            uniformOSMain: getArmorStandScore('AlienEvo.UniformOSMain') ?? 0,
            uniformOSSecondary: getArmorStandScore('AlienEvo.UniformOSSecondary') ?? 0,
            phase10k: getArmorStandScore('AlienEvo.10kPhase') ?? 0,
            stage10k: getArmorStandScore('AlienEvo.10kStage') ?? 0,
            basePrimary: getArmorStandScore('AlienEvo.BasePrimary') ?? -1,
            baseSecondary: getArmorStandScore('AlienEvo.BaseSecondary') ?? -1,
            baseTertiary: getArmorStandScore('AlienEvo.BaseTertiary') ?? -1,
            buttons: getArmorStandScore('AlienEvo.Buttons') ?? -1,
            coreSide: getArmorStandScore('AlienEvo.CoreSide') ?? -1,
            coreTop: getArmorStandScore('AlienEvo.CoreTop') ?? -1,
            dialOuter: getArmorStandScore('AlienEvo.DialOuter') ?? -1,
            dialInner: getArmorStandScore('AlienEvo.DialInner') ?? -1,
            tubes: getArmorStandScore('AlienEvo.Tubes') ?? -1,
            codePosition: getArmorStandScore('code_position') ?? 0
        };
        ('Collected scores:', JSON.stringify(scores));

        // Function to get a property from the armor stand
        let getArmorStandProperty = (propName, defaultValue) => {
            try {
                // Tag the armor stand for identification
                event.server.runCommandSilent(
                    `execute positioned ${pos.x} ${pos.y} ${pos.z} run tag @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] add prop_target`
                );
                
                // Try to get entities with the tag
                let armorStand = null;
                
                // Try multiple methods to get a reference
                try {
                    if (event.level && typeof event.level.getEntities === 'function') {
                        let entities = event.level.getEntities();
                        for (let entity of entities) {
                            if (entity.getTags().contains('prop_target')) {
                                armorStand = entity;
                                break;
                            }
                        }
                    }
                    
                    // Additional methods omitted for brevity - they follow the same pattern
                    
                    // Get property if we have an entity reference
                    if (armorStand) {
                        let propValue = palladium.getProperty(armorStand, propName);
                        
                        // Remove the tag
                        event.server.runCommandSilent(`tag @e[tag=prop_target] remove prop_target`);
                        
                        return propValue || defaultValue;
                    }
                } catch (e) {
                    ('Error getting property ' + propName + ':', e);
                }
                
                // Remove the tag regardless of success
                event.server.runCommandSilent(`tag @e[tag=prop_target] remove prop_target`);
                
                return defaultValue;
            } catch (e) {
                ('Error in getArmorStandProperty for ' + propName + ':', e);
                return defaultValue;
            }
        };

        // Get palladium properties
        let properties = {
            color: getArmorStandProperty('AlienEvo.Color', 'default'),
            primaryColor: getArmorStandProperty('AlienEvo.PrimaryColor', 'default'),
            secondaryColor: getArmorStandProperty('AlienEvo.SecondaryColor', 'default'),
            omnitrixCycle: getArmorStandProperty('omnitrix_cycle', '1'),
            code: getArmorStandProperty('code', '000000')
        };
        ('Retrieved properties:', JSON.stringify(properties));

        // Check for feature tags on armor stand
        ('Checking for armor stand tags...');
        let hasMasterControl = event.server.runCommandSilent(
            `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} if entity @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,tag=AlienEvo.MasterControl]`
        );
        let hasGlowingEyes = event.server.runCommandSilent(
            `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} if entity @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,tag=AlienEvo.GlowingEyes]`
        );
        let hasMainUniform = event.server.runCommandSilent(
            `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} if entity @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,tag=AlienEvo.MainUniform]`
        );
        let hasSecondaryUniform = event.server.runCommandSilent(
            `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} if entity @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,tag=AlienEvo.SecondaryUniform]`
        );
        let hasBaseForm = event.server.runCommandSilent(
            `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} if entity @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,tag=AlienEvo.BaseForm]`
        );

        // Collect DNA from interaction entity
        let allDNA = {};
        let storedAlienCount = 0;
        let alienNamespaceData = "{}";

        // Find the omnitrix interaction entity
        let omnitrixEntity = event.level.getEntitiesWithin(AABB.of(pos.x, pos.y, pos.z, pos.x + 1, pos.y + 2, pos.z + 1))
            .find(entity => entity.type === 'minecraft:interaction' && entity.getTags().contains('omnitrix_interaction'));

        if (omnitrixEntity) {
            ('Found omnitrix interaction entity, collecting data');
            
            // Collect DNA
            for (let playlist = 1; playlist <= 10; playlist++) {
                allDNA[playlist] = {};
                for (let slot = 1; slot <= 10; slot++) {
                    let dnaKey = `alienevo.alien_${playlist}_${slot}`;
                    let alienId = omnitrixEntity.persistentData.getInt(dnaKey);
                    
                    if (alienId > 0) {
                        allDNA[playlist][slot] = {
                            alienId: alienId,
                            alienName: getAlienName(alienId)
                        };
                        storedAlienCount++;
                        (`Found valid alien with ID ${alienId} in playlist ${playlist} slot ${slot}`);
                    }
                }
            }
            
            // Get alien namespace data
            if (omnitrixEntity.persistentData.contains('storedAlienNamespaces')) {
                alienNamespaceData = omnitrixEntity.persistentData.getString('storedAlienNamespaces');
                ('Retrieved stored alien namespace data');
            }
            
            // Get current playlist and slot
            let currentPlaylist = omnitrixEntity.persistentData.getInt('current_playlist') || 1;
            let currentAlienSlot = omnitrixEntity.persistentData.getInt('current_alien_slot') || 1;
            ('Retrieved playlist data - currentPlaylist: ' + currentPlaylist + ', currentAlienSlot: ' + currentAlienSlot);
            
            // Get code-related data
            let codeData = {
                code: omnitrixEntity.persistentData.getString('code') || properties.code || '000000',
                position: omnitrixEntity.persistentData.getString('position') || String(scores.codePosition || '0'),
                masterTargetCode: omnitrixEntity.persistentData.getString('masterTargetCode') || '',
                selfDestructTargetCode: omnitrixEntity.persistentData.getString('selfDestructTargetCode') || ''
            };
            ('Retrieved code data - code: ' + codeData.code + 
                        ', position: ' + codeData.position);
                        
            // Get stored name
            let storedItemName = omnitrixEntity.persistentData.getString('storedItemName');
            ('Retrieved stored name from persistentData:', storedItemName);
            
            // Get fallback properties from interaction entity if needed
            if (properties.color === 'default' && omnitrixEntity.persistentData.contains('storedAlienEvoColor')) {
                properties.color = omnitrixEntity.persistentData.getString('storedAlienEvoColor');
            }
            if (properties.primaryColor === 'default' && omnitrixEntity.persistentData.contains('storedAlienEvoPrimaryColor')) {
                properties.primaryColor = omnitrixEntity.persistentData.getString('storedAlienEvoPrimaryColor');
            }
            if (properties.secondaryColor === 'default' && omnitrixEntity.persistentData.contains('storedAlienEvoSecondaryColor')) {
                properties.secondaryColor = omnitrixEntity.persistentData.getString('storedAlienEvoSecondaryColor');
            }
            
            // Convert color property to score for the omnitrix item selection
            let colorScore = 0;
            if (properties.color && properties.color !== 'default') {
                // Map color names to score values
                let colorMap = {
                    'white': 1,
                    'lightgray': 2,
                    'gray': 3,
                    'black': 4,
                    'brown': 5,
                    'red': 6,
                    'orange': 7,
                    'yellow': 8,
                    'green': 9,
                    'cyan': 10,
                    'lightblue': 11,
                    'blue': 12,
                    'purple': 13,
                    'magenta': 14,
                    'pink': 15
                };
                colorScore = colorMap[properties.color] || 0;
                ('Converted color property ' + properties.color + ' to color score ' + colorScore);
            } else {
                // If no property, use the score directly
                colorScore = scores.color || 0;
                ('Using color score directly: ' + colorScore);
            }
            
            // Construct NBT data
            let nbtData = {
                storedColorScore: scores.color,
                storedUniformOSMainScore: scores.uniformOSMain,
                storedUniformOSSecondaryScore: scores.uniformOSSecondary,
                stored10kPhaseScore: scores.phase10k,
                stored10kStageScore: scores.stage10k,
                storedBasePrimaryScore: scores.basePrimary,
                storedBaseSecondaryScore: scores.baseSecondary,
                storedBaseTertiaryScore: scores.baseTertiary,
                storedButtonsScore: scores.buttons,
                storedCoreSideScore: scores.coreSide,
                storedCoreTopScore: scores.coreTop,
                storedDialOuterScore: scores.dialOuter,
                storedDialInnerScore: scores.dialInner,
                storedTubesScore: scores.tubes,
                
                // Store DNA and count
                storedAllDNA: JSON.stringify(allDNA),
                storedAlienCount: storedAlienCount,
                storedAlienNamespaces: alienNamespaceData,
                
                // Store playlist and slot data
                storedCurrentPlaylist: currentPlaylist,
                storedCurrentAlienSlot: currentAlienSlot,
                
                // Store feature tags
                storedMastercontrol: hasMasterControl ? 1 : 0,
                storedGlowingEyes: hasGlowingEyes ? 1 : 0,
                storedMainUniform: hasMainUniform ? 1 : 0,
                storedSecondaryUniform: hasSecondaryUniform ? 1 : 0,
                storedBaseForm: hasBaseForm ? 1 : 0,
                
                // Store palladium properties
                storedColor: properties.color,
                storedPrimaryColor: properties.primaryColor,
                storedSecondaryColor: properties.secondaryColor,
                storedOmnitrixCycle: properties.omnitrixCycle,
                
                // Store code-related data
                storedCode: codeData.code,
                storedPosition: codeData.position,
                storedMasterTargetCode: codeData.masterTargetCode,
                storedSelfDestructTargetCode: codeData.selfDestructTargetCode
            };

            if (storedItemName) {
                nbtData.display = { Name: storedItemName };
            }

            // Get omnitrix variant and create item
            let omnitrixId = getPrototypeOmnitrixItem(colorScore, scores.phase10k, scores.stage10k);
            ('Selected Omnitrix variant:', omnitrixId);

            // Summon the item with all the NBT data
            event.server.runCommandSilent(
                `execute as ${username} positioned ${pos.x + 0.5} ${pos.y + 0.5} ${pos.z + 0.5} run summon minecraft:item ~ ~ ~ {Item:{id:"${omnitrixId}",Count:1b,tag:${JSON.stringify(nbtData)}},PickupDelay:10,Motion:[0.0,0.2,0.0]}`
            );
            ('Dropped Omnitrix on ground');
        } else {
            ('Could not find omnitrix interaction entity, skipping DNA collection');
        }
    }

    // Kill armor stand
    event.server.runCommandSilent(
        `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} run kill @e[type=minecraft:armor_stand,tag=workbench_stand,sort=nearest,limit=1]`
    );
    ('Killed workbench stand');

    // Kill all interaction entities
    const interactionTags = [
        'baseprimary_interaction',
        'basesecondary_interaction',
        'basetertiary_interaction',
        'coreside_interaction',
        'coretop_interaction',
        'tubes_interaction',
        'dialinner_interaction',
        'dialouter_interaction',
        'buttons_interaction',
        'omnitrix_interaction'
    ];

    // Kill each interaction entity type
    interactionTags.forEach(tag => {
        event.server.runCommandSilent(
            `execute as ${username} positioned ${pos.x} ${pos.y} ${pos.z} run kill @e[type=minecraft:interaction,tag=${tag},sort=nearest,limit=1]`
        );
        (`Removed interaction entity with tag: ${tag}`);
    });
});
BlockEvents.placed('alienevo:omnitrix_workbench', event => {
    let pos = event.block.pos;
    let blockStateStr = event.block.blockState.toString();
    let direction = 'south'; // default direction
    let username = event.player.getGameProfile().getName();
    
    if (blockStateStr.includes('facing=north')) direction = 'north';
    else if (blockStateStr.includes('facing=east')) direction = 'east';
    else if (blockStateStr.includes('facing=west')) direction = 'west';
    
    // Function to rotate coordinates based on facing direction
    function rotate(dx, dz, facing) {
        let dx_new, dz_new;
        if (facing === 'north') {
            dx_new = dx;
            dz_new = dz;
        } else if (facing === 'east') {
            dx_new = -dz;
            dz_new = dx;
        } else if (facing === 'south') {
            dx_new = -dx;
            dz_new = -dz;
        } else if (facing === 'west') {
            dx_new = dz;
            dz_new = -dx;
        }
        return {dx: dx_new, dz: dz_new};
    }
    
    // Define the interactions with base positions (relative to block center)
    let interactions = [
        // Base interactions
        {tag: "basesecondary_interaction", dx: 0.0, dy: 0.6, dz: -0.3},
        {tag: "basetertiary_interaction", dx: -0.19, dy: 0.6, dz: -0.3},
        {tag: "baseprimary_interaction", dx: 0.19, dy: 0.6, dz: -0.3},
        // Core interactions
        {tag: "coreside_interaction", dx: 0.0, dy: 0.4, dz: -0.3},
        {tag: "tubes_interaction", dx: -0.19, dy: 0.4, dz: -0.3},
        {tag: "coretop_interaction", dx: 0.19, dy: 0.4, dz: -0.3},
        // Dial and button interactions
        {tag: "dialinner_interaction", dx: 0.0, dy: 0.23, dz: -0.4},
        {tag: "dialouter_interaction", dx: 0.19, dy: 0.23, dz: -0.4},
        {tag: "buttons_interaction", dx: -0.19, dy: 0.23, dz: -0.4}
    ];
    
    // Loop through each interaction and place it correctly
    for (let i of interactions) {
        let rotated = rotate(i.dx, i.dz, direction);
        let x = pos.x + rotated.dx + 0.5;
        let y = pos.y + i.dy;
        let z = pos.z + rotated.dz + 0.5;
        
        event.server.runCommandSilent(`execute as ${username} run summon minecraft:interaction ${x} ${y} ${z} {width:0.1f,height:0.1f,Tags:["${i.tag}"]}`);
    }
    
    // Omnitrix interaction remains the same for all directions
    event.server.runCommandSilent(`execute as ${username} run summon minecraft:interaction ${pos.x + 0.5} ${pos.y + 1} ${pos.z + 0.5} {width:0.5f,height:0.5f,Tags:["omnitrix_interaction"]}`);
});

// Define valid Omnitrix variants
const VALID_OMNITRIX_TYPES = [
    "alienevo:prototype_omnitrix",
    "alienevo:prototype_omnitrix_white",
    "alienevo:prototype_omnitrix_orange",
    "alienevo:prototype_omnitrix_magenta",
    "alienevo:prototype_omnitrix_light_blue",
    "alienevo:prototype_omnitrix_yellow",
    "alienevo:prototype_omnitrix_pink",
    "alienevo:prototype_omnitrix_gray",
    "alienevo:prototype_omnitrix_light_gray",
    "alienevo:prototype_omnitrix_cyan",
    "alienevo:prototype_omnitrix_purple",
    "alienevo:prototype_omnitrix_blue",
    "alienevo:prototype_omnitrix_brown",
    "alienevo:prototype_omnitrix_green",
    "alienevo:prototype_omnitrix_red",
    "alienevo:prototype_omnitrix_black",
    "alienevo:prototype_omnitrix_phase1",
    "alienevo:prototype_omnitrix_phase1_white",
    "alienevo:prototype_omnitrix_phase1_orange",
    "alienevo:prototype_omnitrix_phase1_magenta",
    "alienevo:prototype_omnitrix_phase1_light_blue",
    "alienevo:prototype_omnitrix_phase1_yellow",
    "alienevo:prototype_omnitrix_phase1_pink",
    "alienevo:prototype_omnitrix_phase1_gray",
    "alienevo:prototype_omnitrix_phase1_light_gray",
    "alienevo:prototype_omnitrix_phase1_cyan",
    "alienevo:prototype_omnitrix_phase1_purple",
    "alienevo:prototype_omnitrix_phase1_blue",
    "alienevo:prototype_omnitrix_phase1_brown",
    "alienevo:prototype_omnitrix_phase1_green",
    "alienevo:prototype_omnitrix_phase1_red",
    "alienevo:prototype_omnitrix_phase1_black",
    "alienevo:prototype_omnitrix_phase2",
    "alienevo:prototype_omnitrix_phase2_white",
    "alienevo:prototype_omnitrix_phase2_orange",
    "alienevo:prototype_omnitrix_phase2_magenta",
    "alienevo:prototype_omnitrix_phase2_light_blue",
    "alienevo:prototype_omnitrix_phase2_yellow",
    "alienevo:prototype_omnitrix_phase2_pink",
    "alienevo:prototype_omnitrix_phase2_gray",
    "alienevo:prototype_omnitrix_phase2_light_gray",
    "alienevo:prototype_omnitrix_phase2_cyan",
    "alienevo:prototype_omnitrix_phase2_purple",
    "alienevo:prototype_omnitrix_phase2_blue",
    "alienevo:prototype_omnitrix_phase2_brown",
    "alienevo:prototype_omnitrix_phase2_green",
    "alienevo:prototype_omnitrix_phase2_red",
    "alienevo:prototype_omnitrix_phase2_black",
    "alienevo:prototype_omnitrix_10k",
    "alienevo:prototype_omnitrix_10k_white",
    "alienevo:prototype_omnitrix_10k_orange",
    "alienevo:prototype_omnitrix_10k_magenta",
    "alienevo:prototype_omnitrix_10k_light_blue",
    "alienevo:prototype_omnitrix_10k_yellow",
    "alienevo:prototype_omnitrix_10k_pink",
    "alienevo:prototype_omnitrix_10k_gray",
    "alienevo:prototype_omnitrix_10k_light_gray",
    "alienevo:prototype_omnitrix_10k_cyan",
    "alienevo:prototype_omnitrix_10k_purple",
    "alienevo:prototype_omnitrix_10k_blue",
    "alienevo:prototype_omnitrix_10k_brown",
    "alienevo:prototype_omnitrix_10k_green",
    "alienevo:prototype_omnitrix_10k_red",
    "alienevo:prototype_omnitrix_10k_black"
];
ItemEvents.entityInteracted('*', event => {
    let username = event.player.getGameProfile().getName();
    
    if (event.target.type != 'minecraft:interaction' || !event.target.getTags().contains('omnitrix_interaction')) {
        return;
    }
    
    var x = Math.floor(event.target.x);
    var y = Math.floor(event.target.y);
    var z = Math.floor(event.target.z);
    
    var mainHandItem = event.player.getMainHandItem();
    if (!mainHandItem || !VALID_OMNITRIX_TYPES.includes(mainHandItem.id)) {
        return;
    }

    if (event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,distance=..1.1,palladium.power=alienevo:omnitrix_workbench]`
    )) {
        return;
    }

    event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run playsound minecraft:item.armor.equip_netherite master @a[distance=..10] ~ ~ ~ 1 1.5`);

    var nbt = mainHandItem.nbt;
    
    // Handle item name
    if (nbt && nbt.display && nbt.display.Name) {
        event.target.persistentData.putString('storedItemName', nbt.display.Name);
    }

    const MAX_PLAYLISTS = 10;
    const SLOTS_PER_PLAYLIST = 10;
    
    try {
        var storedDNA = nbt && nbt.storedAllDNA ? nbt.storedAllDNA : '';
        
        if (storedDNA) {
            var dnaData = JSON.parse(storedDNA);
            
            for (let playlist = 1; playlist <= MAX_PLAYLISTS; playlist++) {
                for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
                    let alienKey = `alienevo.alien_${playlist}_${slot}`;
                    let alienId = 0;
                    
                    if (dnaData[playlist] && dnaData[playlist][slot] && dnaData[playlist][slot].alienId) {
                        alienId = dnaData[playlist][slot].alienId;
                    }
                    
                    event.target.persistentData.putInt(alienKey, alienId);
                }
            }
        } else {
            // Initialize with first 10 aliens in random order (whether or not we have NBT)
            let firstTenAlienIds = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
            
            for (let i = firstTenAlienIds.length - 1; i > 0; i--) {
                let j = Math.floor(Math.random() * (i + 1));
                [firstTenAlienIds[i], firstTenAlienIds[j]] = [firstTenAlienIds[j], firstTenAlienIds[i]];
            }
            
            let playlist = 1;
            for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
                let alienKey = `alienevo.alien_${playlist}_${slot}`;
                let alienId = slot <= firstTenAlienIds.length ? firstTenAlienIds[slot - 1] : 0;
                event.target.persistentData.putInt(alienKey, alienId);
            }
            
            for (let p = 2; p <= MAX_PLAYLISTS; p++) {
                for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
                    let alienKey = `alienevo.alien_${p}_${slot}`;
                    event.target.persistentData.putInt(alienKey, 0);
                }
            }
        }
    } catch (e) {
        // Silent error handling - initialize with default aliens
        let firstTenAlienIds = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        
        for (let i = firstTenAlienIds.length - 1; i > 0; i--) {
            let j = Math.floor(Math.random() * (i + 1));
            [firstTenAlienIds[i], firstTenAlienIds[j]] = [firstTenAlienIds[j], firstTenAlienIds[i]];
        }
        
        let playlist = 1;
        for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
            let alienKey = `alienevo.alien_${playlist}_${slot}`;
            let alienId = slot <= firstTenAlienIds.length ? firstTenAlienIds[slot - 1] : 0;
            event.target.persistentData.putInt(alienKey, alienId);
        }
        
        for (let p = 2; p <= MAX_PLAYLISTS; p++) {
            for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
                let alienKey = `alienevo.alien_${p}_${slot}`;
                event.target.persistentData.putInt(alienKey, 0);
            }
        }
    }

    // Extract color from item ID if no NBT
    let itemColor = 'default';
    if (!nbt) {
        let itemId = mainHandItem.id;
        // Check if the item has a color suffix
        if (itemId !== 'alienevo:prototype_omnitrix') {
            // Handle the special case of 10k omnitrix
            if (itemId.includes('_10k')) {
                // Map to extract color from 10k omnitrix variants
                let colorMap = {
                    'alienevo:prototype_omnitrix_10k': 'default',
                    'alienevo:prototype_omnitrix_10k_white': 'white',
                    'alienevo:prototype_omnitrix_10k_light_gray': 'lightgray',
                    'alienevo:prototype_omnitrix_10k_gray': 'gray',
                    'alienevo:prototype_omnitrix_10k_black': 'black',
                    'alienevo:prototype_omnitrix_10k_brown': 'brown',
                    'alienevo:prototype_omnitrix_10k_red': 'red',
                    'alienevo:prototype_omnitrix_10k_orange': 'orange',
                    'alienevo:prototype_omnitrix_10k_yellow': 'yellow',
                    'alienevo:prototype_omnitrix_10k_green': 'green',
                    'alienevo:prototype_omnitrix_10k_cyan': 'cyan',
                    'alienevo:prototype_omnitrix_10k_light_blue': 'lightblue',
                    'alienevo:prototype_omnitrix_10k_blue': 'blue',
                    'alienevo:prototype_omnitrix_10k_purple': 'purple',
                    'alienevo:prototype_omnitrix_10k_magenta': 'magenta',
                    'alienevo:prototype_omnitrix_10k_pink': 'pink'
                };
                itemColor = colorMap[itemId] || 'default';
            } else {
                // Regular omnitrix handling
                // Extract color from the item ID
                let parts = itemId.split('_');
                if (parts.length > 2) {
                    // The last part should be the color
                    itemColor = parts[parts.length - 1];
                    
                    // Handle special cases like "light_blue" and "light_gray" 
                    if (parts.length > 3 && (parts[parts.length - 2] === "light")) {
                        itemColor = parts[parts.length - 2] + parts[parts.length - 1];
                    } else if (itemColor.startsWith("light")) {
                        itemColor = itemColor;
                    }
                }
            }
        }
    }

    // Store code-related data
    try {
        // Get code information from NBT or use defaults
        let storedCode = nbt && nbt.storedCode ? nbt.storedCode : '000000';
        let storedPosition = nbt && nbt.storedPosition ? nbt.storedPosition : '0';
        let storedMasterTargetCode = nbt && nbt.storedMasterTargetCode ? nbt.storedMasterTargetCode : '';
        let storedSelfDestructTargetCode = nbt && nbt.storedSelfDestructTargetCode ? nbt.storedSelfDestructTargetCode : '';
        
        // Store code data in the interaction entity's persistentData
        event.target.persistentData.putString('code', storedCode);
        event.target.persistentData.putString('position', storedPosition);
        event.target.persistentData.putString('masterTargetCode', storedMasterTargetCode);
        event.target.persistentData.putString('selfDestructTargetCode', storedSelfDestructTargetCode);
    } catch (e) {
        // Silent error handling - use defaults
        event.target.persistentData.putString('code', '000000');
        event.target.persistentData.putString('position', '0');
        event.target.persistentData.putString('masterTargetCode', '');
        event.target.persistentData.putString('selfDestructTargetCode', '');
    }

    // Store namespace data if available
    try {
        if (nbt && nbt.storedAlienNamespaces) {
            event.target.persistentData.putString('storedAlienNamespaces', nbt.storedAlienNamespaces);
        }
    } catch (e) {
        // Silent error handling
    }

    // Store playlist and slot data
    try {
        let storedCurrentPlaylist = nbt && nbt.storedCurrentPlaylist ? nbt.storedCurrentPlaylist : 1;
        let storedCurrentAlienSlot = nbt && nbt.storedCurrentAlienSlot ? nbt.storedCurrentAlienSlot : 1;
        
        event.target.persistentData.putInt('current_playlist', storedCurrentPlaylist);
        event.target.persistentData.putInt('current_alien_slot', storedCurrentAlienSlot);
    } catch (e) {
        // Silent error handling - use defaults
        event.target.persistentData.putInt('current_playlist', 1);
        event.target.persistentData.putInt('current_alien_slot', 1);
    }

    // Tag the armor stand for identification
    event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run tag @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,distance=..1.1] add temp_target`);
    
    // Try multiple methods to get a reference to the tagged armor stand
    let armorStand = null;
    
    try {
        // Method 1: Try using event.level
        if (!armorStand && event.level && typeof event.level.getEntities === 'function') {
            let entities = event.level.getEntities();
            for (let entity of entities) {
                if (entity.getTags().contains('temp_target')) {
                    armorStand = entity;
                    break;
                }
            }
        }
        
        // Method 2: Try using Utils.server
        if (!armorStand && Utils && Utils.server) {
            let entities = Utils.server.getEntities('@e[tag=temp_target]');
            if (entities && entities.length > 0) {
                armorStand = entities[0];
            }
        }
        
        // Method 3: Try using global
        if (!armorStand && global && typeof global.getEntities === 'function') {
            let entities = global.getEntities('@e[tag=temp_target]');
            if (entities && entities.length > 0) {
                armorStand = entities[0];
            }
        }
        
        // Method 4: Try to directly use event.player
        if (!armorStand && typeof event.player.getEntitiesWithinRadius === 'function') {
            let nearbyEntities = event.player.getEntitiesWithinRadius(5);
            for (let entity of nearbyEntities) {
                if (entity.getTags().contains('temp_target')) {
                    armorStand = entity;
                    break;
                }
            }
        }
    } catch (e) {
        // Silent error handling
    }
    
    // Remove the temporary tag
    event.server.runCommandSilent(`tag @e[tag=temp_target] remove temp_target`);
    
    // Set properties if we have a reference
    if (armorStand) {
        try {
            // Set color property from NBT or from item ID
            if (nbt && nbt.storedColor) {
                palladium.setProperty(armorStand, 'AlienEvo.Color', nbt.storedColor);
            } else if (itemColor !== 'default') {
                palladium.setProperty(armorStand, 'AlienEvo.Color', itemColor);
            }
            
            // Set other properties from NBT if available
            if (nbt && nbt.storedPrimaryColor) palladium.setProperty(armorStand, 'AlienEvo.PrimaryColor', nbt.storedPrimaryColor);
            if (nbt && nbt.storedSecondaryColor) palladium.setProperty(armorStand, 'AlienEvo.SecondaryColor', nbt.storedSecondaryColor);
            if (nbt && nbt.storedOmnitrixCycle) palladium.setProperty(armorStand, 'omnitrix_cycle', nbt.storedOmnitrixCycle);
            
            // Set code property if available
            if (nbt && nbt.storedCode) palladium.setProperty(armorStand, 'code', nbt.storedCode);
        } catch (e) {
            // Silent error handling
        }
    } else {
        // Store in interaction entity as fallback
        if (nbt && nbt.storedColor) {
            event.target.persistentData.putString('storedAlienEvoColor', nbt.storedColor);
        } else if (itemColor !== 'default') {
            event.target.persistentData.putString('storedAlienEvoColor', itemColor);
        }
        
        if (nbt && nbt.storedPrimaryColor) event.target.persistentData.putString('storedAlienEvoPrimaryColor', nbt.storedPrimaryColor);
        if (nbt && nbt.storedSecondaryColor) event.target.persistentData.putString('storedAlienEvoSecondaryColor', nbt.storedSecondaryColor);
        if (nbt && nbt.storedOmnitrixCycle) event.target.persistentData.putInt('storedOmnitrixCycle', nbt.storedOmnitrixCycle);
    }

    // Helper function to get NBT integer or use default
    function getStoredScore(nbtTag, defaultValue) {
        return (nbt && nbt.contains(nbtTag)) ? nbt.getInt(nbtTag) : defaultValue;
    }

    // Set 10k phase and stage if it's a 10k watch with no NBT
    if (!nbt && mainHandItem.id.includes('_10k')) {
        event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} as @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] run scoreboard players set @s AlienEvo.10kPhase 1`);
        event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} as @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] run scoreboard players set @s AlienEvo.10kStage 2`);
    } else {
        // Normal score setting
        var scoreCommands = [
            ['Color', 'storedColorScore', 0],
            ['10kPhase', 'stored10kPhaseScore', 0],
            ['10kStage', 'stored10kStageScore', 0],
            ['BasePrimary', 'storedBasePrimaryScore', -1],
            ['BaseSecondary', 'storedBaseSecondaryScore', -1],
            ['BaseTertiary', 'storedBaseTertiaryScore', -1],
            ['Buttons', 'storedButtonsScore', -1],
            ['CoreSide', 'storedCoreSideScore', -1],
            ['CoreTop', 'storedCoreTopScore', -1],
            ['DialInner', 'storedDialInnerScore', -1],
            ['DialOuter', 'storedDialOuterScore', -1],
            ['Tubes', 'storedTubesScore', -1]
        ];

        for (var i = 0; i < scoreCommands.length; i++) {
            var scoreName = scoreCommands[i][0];
            var nbtTag = scoreCommands[i][1];
            var defaultValue = scoreCommands[i][2];
            var scoreValue = getStoredScore(nbtTag, defaultValue);
            event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} as @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] run scoreboard players set @s AlienEvo.${scoreName} ${scoreValue}`);
        }
    }
    
    // Set code position score
    let positionValue = getStoredScore('storedPosition', 0);
    if (nbt && nbt.storedPosition && !isNaN(parseInt(nbt.storedPosition))) {
        positionValue = parseInt(nbt.storedPosition);
    }
    event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} as @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] run scoreboard players set @s code_position ${positionValue}`);

    // Handle MasterControl tag
    if (nbt && nbt.storedMastercontrol) {
        event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run tag @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] add AlienEvo.MasterControl`);
    }
    
    // Handle BaseForm tag
    if (nbt && nbt.storedBaseForm) {
        event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run tag @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] add AlienEvo.BaseForm`);
    }

    // Set superpower and consume item
    event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run superpower set alienevo:omnitrix_workbench @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1]`);
    mainHandItem.count--;
    event.cancel();
});
// Function to get the correct Prototype Omnitrix item based on color score and phase
ItemEvents.entityInteracted(event => {
    let username = event.player.getGameProfile().getName();
    
    // Initial validation
    if (event.target.type != 'minecraft:interaction') {
        ('Target is not an interaction entity');
        return;
    }
    
    if (!event.target.getTags().contains('omnitrix_interaction')) {
        ('Target does not have omnitrix_interaction tag');
        return;
    }

    let x = Math.floor(event.target.x);
    let y = Math.floor(event.target.y);
    let z = Math.floor(event.target.z);
    ('Interaction coordinates:', x, y, z);

    let mainHandItem = event.player.getMainHandItem();
    if (mainHandItem.id !== 'minecraft:air') {
        ('Hand is not empty:', mainHandItem.id);
        return;
    }

    // Check for valid workbench stand
    let hasWorkbenchPower = event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,distance=0..1.1,sort=nearest,limit=1,palladium.power=alienevo:omnitrix_workbench]`
    );
    
    if (!hasWorkbenchPower) {
        ('No valid workbench stand found, exiting');
        return;
    }

    // DNA Collection from interaction entity
    let allDNA = {};
    let storedAlienCount = 0;

    ('Starting DNA collection from interaction entity...');
    for (let playlist = 1; playlist <= 10; playlist++) {
        allDNA[playlist] = {};
        for (let slot = 1; slot <= 10; slot++) {
            let dnaKey = `alienevo.alien_${playlist}_${slot}`;
            let alienId = event.target.persistentData.getInt(dnaKey);
            
            ('Checking alien at ' + dnaKey + ':', 'rawId: ' + alienId + ', isValid: ' + (alienId !== null && alienId > 0));

            if (alienId > 0) {
                allDNA[playlist][slot] = {
                    alienId: alienId,
                    alienName: getAlienName(alienId)
                };
                storedAlienCount++;
                ('Found valid alien with ID ' + alienId + ' in playlist ' + playlist + ' slot ' + slot);
                event.target.persistentData.putInt(dnaKey, 0);
                ('Cleared alien ID from ' + dnaKey);
            }
        }
    }

    // Retrieve alien namespace data if available
    let alienNamespaceData = "{}";
    if (event.target.persistentData.contains('storedAlienNamespaces')) {
        alienNamespaceData = event.target.persistentData.getString('storedAlienNamespaces');
        ('Retrieved stored alien namespace data');
        event.target.persistentData.remove('storedAlienNamespaces');
    }

    // Helper function for getting armor stand scores
    function getArmorStandScore(scoreName) {
        let cmd = `scoreboard players get @e[type=armor_stand,distance=..1.1,tag=workbench_stand,sort=nearest,limit=1] ${scoreName}`;
        let scoreResult = event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run ${cmd}`);
        return isNaN(scoreResult) ? null : parseInt(scoreResult)
    }

    // Get all scores with default values
    let scores = {
        color: getArmorStandScore('AlienEvo.Color') ?? 0,
        uniformOSMain: getArmorStandScore('AlienEvo.UniformOSMain') ?? 0,
        uniformOSSecondary: getArmorStandScore('AlienEvo.UniformOSSecondary') ?? 0,
        phase10k: getArmorStandScore('AlienEvo.10kPhase') ?? 0,
        stage10k: getArmorStandScore('AlienEvo.10kStage') ?? 0,
        basePrimary: getArmorStandScore('AlienEvo.BasePrimary') ?? -1,
        baseSecondary: getArmorStandScore('AlienEvo.BaseSecondary') ?? -1,
        baseTertiary: getArmorStandScore('AlienEvo.BaseTertiary') ?? -1,
        buttons: getArmorStandScore('AlienEvo.Buttons') ?? -1,
        coreSide: getArmorStandScore('AlienEvo.CoreSide') ?? -1,
        coreTop: getArmorStandScore('AlienEvo.CoreTop') ?? -1,
        dialOuter: getArmorStandScore('AlienEvo.DialOuter') ?? -1,
        dialInner: getArmorStandScore('AlienEvo.DialInner') ?? -1,
        tubes: getArmorStandScore('AlienEvo.Tubes') ?? -1,
        codePosition: getArmorStandScore('code_position') ?? 0
    };
    ('Collected scores:', JSON.stringify(scores));

    // Get properties using palladium 
    let getArmorStandProperty = (propName, defaultValue) => {
        try {
            // Tag the armor stand for identification
            event.server.runCommandSilent(
                `execute positioned ${x} ${y} ${z} run tag @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1,distance=..1.1] add prop_target`
            );
            
            // Try to get entities with the tag
            let armorStand = null;
            
            // Try multiple methods to get a reference
            try {
                if (event.level && typeof event.level.getEntities === 'function') {
                    let entities = event.level.getEntities();
                    for (let entity of entities) {
                        if (entity.getTags().contains('prop_target')) {
                            armorStand = entity;
                            break;
                        }
                    }
                }
                
                if (!armorStand && Utils && Utils.server) {
                    let entities = Utils.server.getEntities('@e[tag=prop_target]');
                    if (entities && entities.length > 0) {
                        armorStand = entities[0];
                    }
                }
                
                if (!armorStand && global && typeof global.getEntities === 'function') {
                    let entities = global.getEntities('@e[tag=prop_target]');
                    if (entities && entities.length > 0) {
                        armorStand = entities[0];
                    }
                }
                
                if (!armorStand && typeof event.player.getEntitiesWithinRadius === 'function') {
                    let nearbyEntities = event.player.getEntitiesWithinRadius(5);
                    for (let entity of nearbyEntities) {
                        if (entity.getTags().contains('prop_target')) {
                            armorStand = entity;
                            break;
                        }
                    }
                }
                
                // Get property if we have an entity reference
                if (armorStand) {
                    let propValue = palladium.getProperty(armorStand, propName);
                    
                    // Remove the tag
                    event.server.runCommandSilent(`tag @e[tag=prop_target] remove prop_target`);
                    
                    return propValue || defaultValue;
                }
            } catch (e) {
                ('Error getting property ' + propName + ':', e);
            }
            
            // Remove the tag regardless of success
            event.server.runCommandSilent(`tag @e[tag=prop_target] remove prop_target`);
            
            // Fallback: check if stored in interaction entity's persistentData
            if (event.target.persistentData.contains('stored' + propName.replace(/\./g, ''))) {
                return event.target.persistentData.getString('stored' + propName.replace(/\./g, ''));
            }
            
            return defaultValue;
        } catch (e) {
            ('Error in getArmorStandProperty for ' + propName + ':', e);
            return defaultValue;
        }
    };

    // Get palladium properties
    let properties = {
        color: getArmorStandProperty('AlienEvo.Color', 'default'),
        primaryColor: getArmorStandProperty('AlienEvo.PrimaryColor', 'default'),
        secondaryColor: getArmorStandProperty('AlienEvo.SecondaryColor', 'default'),
        omnitrixCycle: getArmorStandProperty('omnitrix_cycle', '1'),
        code: getArmorStandProperty('code', '000000')
    };
    ('Retrieved properties:', JSON.stringify(properties));

    // Check for feature tags on armor stand
    ('Checking for armor stand tags...');
    let hasMasterControl = event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,distance=0..1.1,sort=nearest,limit=1,tag=AlienEvo.MasterControl]`
    );
    let hasGlowingEyes = event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,distance=0..1.1,sort=nearest,limit=1,tag=AlienEvo.GlowingEyes]`
    );
    let hasMainUniform = event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,distance=0..1.1,sort=nearest,limit=1,tag=AlienEvo.MainUniform]`
    );
    let hasSecondaryUniform = event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,distance=0..1.1,sort=nearest,limit=1,tag=AlienEvo.SecondaryUniform]`
    );
    let hasBaseForm = event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} if entity @e[type=armor_stand,tag=workbench_stand,distance=0..1.1,sort=nearest,limit=1,tag=AlienEvo.BaseForm]`
    );

    ('Armor stand tags - MasterControl: ' + hasMasterControl + 
                ', GlowingEyes: ' + hasGlowingEyes + 
                ', MainUniform: ' + hasMainUniform + 
                ', SecondaryUniform: ' + hasSecondaryUniform + 
                ', BaseForm: ' + hasBaseForm);

    // Retrieve current playlist and slot information
    let currentPlaylist = event.target.persistentData.getInt('current_playlist') || 1;
    let currentAlienSlot = event.target.persistentData.getInt('current_alien_slot') || 1;
    ('Retrieved playlist data - currentPlaylist: ' + currentPlaylist + ', currentAlienSlot: ' + currentAlienSlot);
    event.target.persistentData.remove('current_playlist');
    event.target.persistentData.remove('current_alien_slot');

    // Retrieve code-related data
    let codeData = {
        code: event.target.persistentData.getString('code') || '000000',
        position: event.target.persistentData.getString('position') || String(scores.codePosition || '0'),
        masterTargetCode: event.target.persistentData.getString('masterTargetCode') || '',
        selfDestructTargetCode: event.target.persistentData.getString('selfDestructTargetCode') || ''
    };
    ('Retrieved code data - code: ' + codeData.code + 
                ', position: ' + codeData.position + 
                ', masterTargetCode: ' + codeData.masterTargetCode + 
                ', selfDestructTargetCode: ' + codeData.selfDestructTargetCode);
    
    // Clear code data from interaction entity
    event.target.persistentData.remove('code');
    event.target.persistentData.remove('position');
    event.target.persistentData.remove('masterTargetCode');
    event.target.persistentData.remove('selfDestructTargetCode');

    // Convert color property to score for the omnitrix item selection
    let colorScore = 0;
    if (properties.color && properties.color !== 'default') {
        // Map color names to score values
        let colorMap = {
            'white': 1,
            'lightgray': 2,
            'gray': 3,
            'black': 4,
            'brown': 5,
            'red': 6,
            'orange': 7,
            'yellow': 8,
            'green': 9,
            'cyan': 10,
            'lightblue': 11,
            'blue': 12,
            'purple': 13,
            'magenta': 14,
            'pink': 15
        };
        colorScore = colorMap[properties.color] || 0;
        ('Converted color property ' + properties.color + ' to color score ' + colorScore);
    } else {
        // If no property, try to use the score directly
        colorScore = scores.color || 0;
        ('Using color score directly: ' + colorScore);
    }

    let nbtData = {
        storedColorScore: scores.color,
        storedUniformOSMainScore: scores.uniformOSMain,
        storedUniformOSSecondaryScore: scores.uniformOSSecondary,
        stored10kPhaseScore: scores.phase10k,
        stored10kStageScore: scores.stage10k,
        storedBasePrimaryScore: scores.basePrimary,
        storedBaseSecondaryScore: scores.baseSecondary,
        storedBaseTertiaryScore: scores.baseTertiary,
        storedButtonsScore: scores.buttons,
        storedCoreSideScore: scores.coreSide,
        storedCoreTopScore: scores.coreTop,
        storedDialOuterScore: scores.dialOuter,
        storedDialInnerScore: scores.dialInner,
        storedTubesScore: scores.tubes,
        
        // Store DNA and count
        storedAllDNA: JSON.stringify(allDNA),
        storedAlienCount: storedAlienCount,
        storedAlienNamespaces: alienNamespaceData,
        
        // Store playlist and slot data
        storedCurrentPlaylist: currentPlaylist,
        storedCurrentAlienSlot: currentAlienSlot,
        
        // Store feature tags
        storedMastercontrol: hasMasterControl ? 1 : 0,
        storedGlowingEyes: hasGlowingEyes ? 1 : 0,
        storedMainUniform: hasMainUniform ? 1 : 0,
        storedSecondaryUniform: hasSecondaryUniform ? 1 : 0,
        storedBaseForm: hasBaseForm ? 1 : 0,
        
        // Store palladium properties
        storedColor: properties.color,
        storedPrimaryColor: properties.primaryColor,
        storedSecondaryColor: properties.secondaryColor,
        storedOmnitrixCycle: properties.omnitrixCycle,
        
        // Store code-related data
        storedCode: codeData.code,
        storedPosition: codeData.position,
        storedMasterTargetCode: codeData.masterTargetCode,
        storedSelfDestructTargetCode: codeData.selfDestructTargetCode
    };

    // Handle name retrieval
    let storedName = event.target.persistentData.getString('storedItemName');
    ('Retrieved stored name:', storedName);

    if (storedName) {
        nbtData.display = { Name: storedName };
        event.target.persistentData.remove('storedItemName');
        ('Applied stored name to NBT:', storedName);
    }

    // Get omnitrix variant and create item
    let omnitrixId = getPrototypeOmnitrixItem(colorScore, scores.phase10k, scores.stage10k);
    ('Selected Omnitrix variant:', omnitrixId);

    // Create and give the omnitrix
    let omnitrix = Item.of(omnitrixId);
    omnitrix.setNbt(nbtData);
    event.player.give(omnitrix);
    ('Gave Omnitrix to player');

    // Play retrieval sound
    event.server.runCommandSilent(`execute positioned ${x} ${y} ${z} run playsound minecraft:item.armor.equip_leather master @a[distance=..10] ~ ~ ~ 1 1`);
    ('Played retrieval sound');

    // Remove workbench superpower and cleanup tags
    event.server.runCommandSilent(
        `execute as ${username} positioned ${x} ${y} ${z} run superpower remove alienevo:omnitrix_workbench @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1]`
    );

    // Remove all tags
    ['MasterControl', 'GlowingEyes', 'MainUniform', 'SecondaryUniform', 'BaseForm'].forEach(tag => {
        event.server.runCommandSilent(
            `execute as ${username} positioned ${x} ${y} ${z} run tag @e[type=armor_stand,tag=workbench_stand,sort=nearest,limit=1] remove AlienEvo.${tag}`
        );
    });
    ('Removed all tags and superpower from armor stand');

    event.cancel();
    ('Omnitrix retrieval process complete');
});
// Helper function to get alien name (implement as needed)
function getAlienName(alienId) {
    return `Alien_${alienId}`;
}

function getPrototypeOmnitrixItem(colorScore, phase, stage) {
    // Check for 10K
    if (phase === 1 && stage === 2) {
        const omnitrix10KMap = {
            0: 'alienevo:prototype_omnitrix_10k',
            1: 'alienevo:prototype_omnitrix_10k_white',
            2: 'alienevo:prototype_omnitrix_10k_light_gray',
            3: 'alienevo:prototype_omnitrix_10k_gray',
            4: 'alienevo:prototype_omnitrix_10k_black',
            5: 'alienevo:prototype_omnitrix_10k_brown',
            6: 'alienevo:prototype_omnitrix_10k_red',
            7: 'alienevo:prototype_omnitrix_10k_orange',
            8: 'alienevo:prototype_omnitrix_10k_yellow',
            9: 'alienevo:prototype_omnitrix_10k_green',
            10: 'alienevo:prototype_omnitrix_10k_cyan',
            11: 'alienevo:prototype_omnitrix_10k_light_blue',
            12: 'alienevo:prototype_omnitrix_10k_blue',
            13: 'alienevo:prototype_omnitrix_10k_purple',
            14: 'alienevo:prototype_omnitrix_10k_magenta',
            15: 'alienevo:prototype_omnitrix_10k_pink'
        };
        return omnitrix10KMap[colorScore] || 'alienevo:prototype_omnitrix_10k';
    }

    // Check for Phase 2
    if (phase === 1 && stage === 1) {
        const phase2OmnitrixMap = {
            0: 'alienevo:prototype_omnitrix_phase2',
            1: 'alienevo:prototype_omnitrix_phase2_white',
            2: 'alienevo:prototype_omnitrix_phase2_light_gray',
            3: 'alienevo:prototype_omnitrix_phase2_gray',
            4: 'alienevo:prototype_omnitrix_phase2_black',
            5: 'alienevo:prototype_omnitrix_phase2_brown',
            6: 'alienevo:prototype_omnitrix_phase2_red',
            7: 'alienevo:prototype_omnitrix_phase2_orange',
            8: 'alienevo:prototype_omnitrix_phase2_yellow',
            9: 'alienevo:prototype_omnitrix_phase2_green',
            10: 'alienevo:prototype_omnitrix_phase2_cyan',
            11: 'alienevo:prototype_omnitrix_phase2_light_blue',
            12: 'alienevo:prototype_omnitrix_phase2_blue',
            13: 'alienevo:prototype_omnitrix_phase2_purple',
            14: 'alienevo:prototype_omnitrix_phase2_magenta',
            15: 'alienevo:prototype_omnitrix_phase2_pink'
        };
        return phase2OmnitrixMap[colorScore] || 'alienevo:prototype_omnitrix_phase2';
    }
    
    // Check for Phase 1
    if (phase === 1 && stage === 0) {
        const phase1OmnitrixMap = {
            0: 'alienevo:prototype_omnitrix_phase1',
            1: 'alienevo:prototype_omnitrix_phase1_white',
            2: 'alienevo:prototype_omnitrix_phase1_light_gray',
            3: 'alienevo:prototype_omnitrix_phase1_gray',
            4: 'alienevo:prototype_omnitrix_phase1_black',
            5: 'alienevo:prototype_omnitrix_phase1_brown',
            6: 'alienevo:prototype_omnitrix_phase1_red',
            7: 'alienevo:prototype_omnitrix_phase1_orange',
            8: 'alienevo:prototype_omnitrix_phase1_yellow',
            9: 'alienevo:prototype_omnitrix_phase1_green',
            10: 'alienevo:prototype_omnitrix_phase1_cyan',
            11: 'alienevo:prototype_omnitrix_phase1_light_blue',
            12: 'alienevo:prototype_omnitrix_phase1_blue',
            13: 'alienevo:prototype_omnitrix_phase1_purple',
            14: 'alienevo:prototype_omnitrix_phase1_magenta',
            15: 'alienevo:prototype_omnitrix_phase1_pink'
        };
        return phase1OmnitrixMap[colorScore] || 'alienevo:prototype_omnitrix_phase1';
    }
    
    // Default to original Omnitrix map
    const omnitrixMap = {
        0: 'alienevo:prototype_omnitrix',
        1: 'alienevo:prototype_omnitrix_white',
        2: 'alienevo:prototype_omnitrix_light_gray',
        3: 'alienevo:prototype_omnitrix_gray',
        4: 'alienevo:prototype_omnitrix_black',
        5: 'alienevo:prototype_omnitrix_brown',
        6: 'alienevo:prototype_omnitrix_red',
        7: 'alienevo:prototype_omnitrix_orange',
        8: 'alienevo:prototype_omnitrix_yellow',
        9: 'alienevo:prototype_omnitrix_green',
        10: 'alienevo:prototype_omnitrix_cyan',
        11: 'alienevo:prototype_omnitrix_light_blue',
        12: 'alienevo:prototype_omnitrix_blue',
        13: 'alienevo:prototype_omnitrix_purple',
        14: 'alienevo:prototype_omnitrix_magenta',
        15: 'alienevo:prototype_omnitrix_pink'
    };
    return omnitrixMap[colorScore] || 'alienevo:prototype_omnitrix';
}


